/* xdwapi sample 13

Function :
  Generates an XML file from a DocuWorks file.
  Newly adds the information described in an XML file to a DocuWorks file.

How to use :
  xdwsmpl13 get|set docuWorksDocument xmlDocument

  get : Generates an XML file from a DocuWorks file.
  set : Newly adds the information described in an XML file to a DocuWorks file.
  docuWorksDocument : DocuWorks document
  xmlDocument : XML document
*/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>

#include <xdw_api.h>
#include <xdw_xml.h>

// Display what cause an error.
void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_FILE_EXISTS:
		fprintf(stderr, "File exists.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_ACCESS:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

// Dispaly a help message.
void print_help()
{
	fprintf(stderr, "How to use : xdwsmpl13 get|set docuWorksDocument xmlDocument");
}

// Generates an XML file from a DocuWorks file.
void get_xml(const char* dw_full_path, const char* xml_full_path)
{
	int api_result = 0;

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX open_mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_READONLY, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(dw_full_path, &h, (XDW_OPEN_MODE*)&open_mode);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Use XDW_GetXmlFile to Generate an XML file from a DocuWorks file. 
	XDW_XML_GET_OPTION_DOCUWORKSFILE option = {
		sizeof(XDW_XML_GET_OPTION_DOCUWORKSFILE), XDW_XML_ENCODING_UTF8, 1, 0, 
		XDW_XML_CONVERT_KNOWN_AND_SPECIFIED_CODEPAGE, CP_ACP, 1, 
		XDW_XML_DOCUWORKSFILE_SCHEMA_VERSION_710, XDW_XML_DOCUWORKSFILE_XDWAPI_VERSION_710, NULL};
	api_result = XDW_GetXmlFile(h, xml_full_path, XDW_XML_FORMAT_DOCUWORKSFILE, &option, NULL);

	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Close the document handle.
	api_result = XDW_CloseDocumentHandle(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return;
	}
}

// Newly adds the information described in XML file to DocuWorks file.
void set_xml(const char* dw_full_path, const char* xml_full_path)
{
	int api_result = 0;

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX open_mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(dw_full_path, &h, (XDW_OPEN_MODE*)&open_mode);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Use XDW_SetXMLFile to newly add the information in a specified XML file to a DocuWorks file.
	XDW_XML_ERROR_INFO error_info = {
		sizeof(XDW_XML_ERROR_INFO), 0, 0};
	api_result = XDW_SetXmlFile(h, xml_full_path, XDW_XML_FORMAT_DOCUWORKSFILE, NULL, &error_info, NULL);
	if (api_result < 0) {
		print_error(api_result);
		fprintf(stderr, "XML file has an error located at: Line %d, Column %d.", error_info.nLine, error_info.nColumn);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Reflect the changes to the document.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		XDW_CloseDocumentHandle(h, NULL);
		return;
	}

	// Close the document handle.
	api_result = XDW_CloseDocumentHandle(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return;
	}
	return;
}

int main(int argc, char** argv)
{
	if (argc != 4) {
		print_help();
		return 0;
	}

	char dw_full_path[_MAX_PATH] = {0};
	char xml_full_path[_MAX_PATH] = {0};

	_fullpath(dw_full_path, argv[2], _MAX_PATH);
	_fullpath(xml_full_path, argv[3], _MAX_PATH);

	if (_strcmpi(argv[1], "get") == 0) {
		get_xml(dw_full_path, xml_full_path);
	}
	else if (_strcmpi(argv[1], "set") == 0) {
		set_xml(dw_full_path, xml_full_path);
	}
	else {
		print_help();
	}
	
	return 0;
}
