/* xdwapi sample 12

Function :
    Verifies signatures in the DocuWorks document, and acquires those information.
How to use :
    xdwsmpl12 file
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl12
{
    class Program
    {
        static int Main(string[] args)
        {
            if (args.Length != 1)
            {
                print_usage();
                return 0;
            }

            // Open the document handle.
            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_READONLY;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;
            int api_result = Xdwapi.XDW_OpenDocumentHandle(args[0], ref handle, mode);

            if (api_result == 0)
            {
                // Acquires the number of signatures attached to a DocuWorks file.
                int sig_num = Xdwapi.XDW_GetDocumentSignatureNumber(handle);

                if (sig_num >= 0)
                {
                    Console.WriteLine("The number of signatures : {0}", sig_num);
                    for (int i = 1; i <= sig_num; i++)
                    {
                        // Acquires signature information.
                        check_signature(handle, i);
                    }
                }
                else
                {
                    print_error(api_result);
                }

                // Close the document handle.
                Xdwapi.XDW_CloseDocumentHandle(handle);
            }
            else
            {
                print_error(api_result);
                return 0;
            }

            return 1;
        }

        // Acquires signature infomation.
        static void check_signature(Xdwapi.XDW_DOCUMENT_HANDLE handle, int sig)
        {
            Console.WriteLine("{0}th signature :", sig);

            // Updates signature status.
            Xdwapi.XDW_SIGNATURE_MODULE_STATUS module_status = new Xdwapi.XDW_SIGNATURE_MODULE_STATUS();
            int api_result = Xdwapi.XDW_UpdateSignatureStatus(handle, sig, ref module_status);

            if (api_result != 0)
            {
                // Print XDWAPI error.
                print_error(api_result);
                if (api_result == Xdwapi.XDW_E_SIGNATURE_MODULE)
                {
                    // Print error unique to the signature module.
                    print_module_error(module_status.SignatureType, module_status.ErrorStatus);
                }
            }

            // Acquires signature infomations.
            Xdwapi.XDW_SIGNATURE_INFO_V5 sig_info = new Xdwapi.XDW_SIGNATURE_INFO_V5();
            Object module_info = null;
            api_result = Xdwapi.XDW_GetSignatureInformation(handle, sig, ref sig_info, ref module_info, ref module_status);

            if (api_result == 0)
            {
                print_sig_info(sig_info);
                if (sig_info.SignatureType == Xdwapi.XDW_SIGNATURE_STAMP)
                {
                    Xdwapi.XDW_SIGNATURE_STAMP_INFO_V5 stamp_info = (Xdwapi.XDW_SIGNATURE_STAMP_INFO_V5)module_info;
                    print_sig_stamp_info(stamp_info);
                }
                else if (sig_info.SignatureType == Xdwapi.XDW_SIGNATURE_PKI)
                {
                    Xdwapi.XDW_SIGNATURE_PKI_INFO_V5 pki_info = (Xdwapi.XDW_SIGNATURE_PKI_INFO_V5)module_info;
                    print_sig_pki_info(pki_info);

                    if (pki_info.SignerCert != null)
                    {
                        // Certificate data is saved as a .cer file.
                        string fname = "cert-" + sig.ToString("D4") + ".cer";
                        if (!File.Exists(fname))
                        {
                            FileStream stream = new FileStream(fname, FileMode.CreateNew);
                            BinaryWriter writer = new BinaryWriter(stream);
                            writer.Write(pki_info.SignerCert.Export(X509ContentType.Cert));
                            writer.Close();
                            stream.Close();
                        }
                    }
                }
            }
            else
            {
                print_error(api_result);
                if (api_result == Xdwapi.XDW_E_SIGNATURE_MODULE)
                {
                    print_module_error(module_status.SignatureType, module_status.ErrorStatus);
                }
            }
        }

        // Print informations of the signature module.
        static void print_sig_info(Xdwapi.XDW_SIGNATURE_INFO_V5 sig_info)
        {
            Console.WriteLine("  Signature module : ");
            switch (sig_info.SignatureType)
            {
                case Xdwapi.XDW_SIGNATURE_STAMP:
                    Console.WriteLine("DocuWorks electronic signature");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI:
                    Console.WriteLine("Electronic Certificate");
                    break;
                default:
                    Console.WriteLine("Unknown module");
                    break;
            }
            Console.WriteLine("   Position : {0} page(s). {1}mm from the left. {2}mm from the top.",
                    sig_info.Page, sig_info.HorPos / 100.0, sig_info.VerPos / 100.0);
            Console.WriteLine("   Size : Width {0}mm Height {1}mm",
                    sig_info.Width / 100.0, sig_info.Height / 100.0);

            DateTime signed_time = sig_info.SignedTime;
            Console.WriteLine("   Date and Time: {0}", signed_time.ToLongDateString());
        }

        // Print informations unique to the DocuWorks electronic signature.
        static void print_sig_stamp_info(Xdwapi.XDW_SIGNATURE_STAMP_INFO_V5 stamp_info)
        {
            Console.WriteLine("    Signature Name : {0}", stamp_info.StampName);
            Console.WriteLine("    Owner Name : {0}", stamp_info.OwnerName);

            DateTime valid_date = stamp_info.ValidDate;
            Console.WriteLine("    Expiration Date: {0}", valid_date.ToLongDateString());

            Console.WriteLine("    Remarks : {0}", stamp_info.Remarks);
            Console.WriteLine("    The verification result of the document : ");
            switch (stamp_info.DocVerificationStatus)
            {
                case Xdwapi.XDW_SIGNATURE_STAMP_DOC_NONE:
                    Console.WriteLine("Validity Unknown");
                    break;
                case Xdwapi.XDW_SIGNATURE_STAMP_DOC_NOEDIT:
                    Console.WriteLine("Not modified");
                    break;
                case Xdwapi.XDW_SIGNATURE_STAMP_DOC_EDIT:
                    Console.WriteLine("Modified");
                    break;
                case Xdwapi.XDW_SIGNATURE_STAMP_DOC_BAD:
                    Console.WriteLine("Problem");
                    break;
                default:
                    Console.WriteLine("Unknown status");
                    break;
            }
            Console.WriteLine("    The verification result of the Electronic Signature : ");
            switch (stamp_info.StampVerificationStatus)
            {
                case Xdwapi.XDW_SIGNATURE_STAMP_DOC_NONE:
                    Console.WriteLine("Validity Unknown");
                    break;
                case Xdwapi.XDW_SIGNATURE_STAMP_STAMP_TRUSTED:
                    Console.WriteLine("Trustworthy");
                    break;
                case Xdwapi.XDW_SIGNATURE_STAMP_STAMP_NOTRUST:
                    Console.WriteLine("No trustworthy");
                    break;
                default:
                    Console.WriteLine("Unknown status");
                    break;
            }
        }

        // Print informations unique to the electronic certificate.
        static void print_sig_pki_info(Xdwapi.XDW_SIGNATURE_PKI_INFO_V5 pki_info)
        {
            Console.WriteLine("    Module Name : {0}", pki_info.Module);
            Console.WriteLine("    Signer : {0} ({1})", pki_info.Subject, pki_info.SubjectDN);
            Console.WriteLine("    Certification Authority : {0} ({1})", pki_info.Issuer, pki_info.IssuerDN);
            Console.WriteLine("    Expiration Date : {0} - {1}", pki_info.NotBefore, pki_info.NotAfter);
            Console.WriteLine("    Serial Number : {0}", pki_info.Serial);
            Console.WriteLine("    Remarks : {0}", pki_info.Remarks);
            Console.WriteLine("    Signature time : {0}", pki_info.SigningTime);
            Console.WriteLine("    The verification result of the document : ");
            switch (pki_info.DocVerificationStatus)
            {
                case Xdwapi.XDW_SIGNATURE_PKI_DOC_UNKNOWN:
                    Console.WriteLine("Not verified.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_DOC_GOOD:
                    Console.WriteLine("The DocuWorks file has not been edited. The trustworthiness of the Certificate used for the signature cannot be confirmed.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_DOC_MODIFIED:
                    Console.WriteLine("The DocuWorks file has been edited. The trustworthiness of the Certificate used for the signature cannot be confirmed.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_DOC_BAD:
                    Console.WriteLine("The DocuWorks file has one or more problems. Or the Certificate used for the signature is invalid.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_DOC_GOOD_TRUSTED:
                    Console.WriteLine("The DocuWorks file has not been edited. The trustworthiness of the Certificate used for the signature has been confirmed.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_DOC_MODIFIED_TRUSTED:
                    Console.WriteLine("The DocuWorks file has been edited. The trustworthiness of the Certificate used for the signature has been confirmed.");
                    break;
                default:
                    Console.WriteLine("Unknown status.");
                    break;
            }
            Console.WriteLine("    Verification Level : ");
            switch (pki_info.CertVerificationType)
            {
                case Xdwapi.XDW_SIGNATURE_PKI_TYPE_LOW:
                    Console.WriteLine("The trustworthiness level of the Certificate is low.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_TYPE_MID_LOCAL:
                    Console.WriteLine("The trustworthiness level of the Certificate is middle. Revocation is checked offline.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_TYPE_MID_NETWORK:
                    Console.WriteLine("The trustworthiness level of the Certificate is middle. Revocation is checked online.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_TYPE_HIGH_LOCAL:
                    Console.WriteLine("The trustworthiness level of the Certificate is high. Revocation is checked offline.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_TYPE_HIGH_NETWORK:
                    Console.WriteLine("The trustworthiness level of the Certificate is high. Revocation is checked online.");
                    break;
                default:
                    Console.WriteLine("Unknown level.");
                    break;
            }
            Console.WriteLine("    The verification result of the certificate : ");
            switch (pki_info.CertVerificationStatus)
            {
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_UNKNOWN:
                    Console.WriteLine("Not verified.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_OK:
                    Console.WriteLine("Valid.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_NO_ROOT_CERTIFICATE:
                    Console.WriteLine("Cannot find Root Certificate.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_NO_REVOCATION_CHECK:
                    Console.WriteLine("Failed to confirm revocation.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_OUT_OF_VALIDITY:
                    Console.WriteLine("Outside of the valid date.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_OUT_OF_VALIDITY_AT_SIGNED_TIME:
                    Console.WriteLine("Date of signature is outside of the valid date of the Certificate.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_REVOKE_CERTIFICATE:
                    Console.WriteLine("The Certificate is revoked.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_REVOKE_INTERMEDIATE_CERTIFICATE:
                    Console.WriteLine("The Intermediate Certificate is revoked.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_INVALID_SIGNATURE:
                    Console.WriteLine("The signature of the Certificate is invalid.");
                    break;
                case Xdwapi.XDW_SIGNATURE_PKI_CERT_INVALID_USAGE:
                    Console.WriteLine("The use of the key is invalid.");
                    break;
                default:
                    Console.WriteLine("Unknown status.");
                    break;
            }
        }

        // Print XDWAPI error.
        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code)
            {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                case Xdwapi.XDW_E_SIGNATURE_MODULE:
                    errorWriter.WriteLine("Error unique to the signature module occurred.");
                    break;
                default:
                    errorWriter.WriteLine("Error.");
                    break;
            }
        }

        // Print error unique to the signature module.
        static void print_module_error(int module, int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (module)
            {
                // DocuWorks Electronic Signature
                case Xdwapi.XDW_SIGNATURE_STAMP:
                    errorWriter.WriteLine("DocuWorks Electronic Signature : ");
                    switch (code)
                    {
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_NO_OPENING_CASE:
                            errorWriter.WriteLine("The processing failed because no open Electronic Signature Case exists.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_NO_SELFSTAMP:
                            errorWriter.WriteLine("My Electronic Signature is not available in the Electronic Signature Case.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_OUT_OF_VALIDITY:
                            errorWriter.WriteLine("The Electronic Signature is outside the valid date.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_INVALID_DATA:
                            errorWriter.WriteLine("The processing failed because of uncomformity of data such as document to attach the signature and Electronic Signature Case.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_OUT_OF_MEMORY:
                            errorWriter.WriteLine("The processing failed due to insufficient memory.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_UNKNOWN:
                            errorWriter.WriteLine("The processing failed due to unexpected error.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_OK:
                            errorWriter.WriteLine("The processing succeeded.");
                            break;
                        default:
                            errorWriter.WriteLine("Error.");
                            break;
                    }
                    break;
                // Electronic Certificate
                case Xdwapi.XDW_SIGNATURE_PKI:
                    errorWriter.WriteLine("Electronic Certificate module : ");
                    switch (code)
                    {
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_PLATFORM:
                            errorWriter.WriteLine("Executing environment is inappropriate.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_WRITE_REG_ERROR:
                            errorWriter.WriteLine("Writing operation to the registry failed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_TRUST_LEVEL:
                            errorWriter.WriteLine("The set trustworthiness level is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_REVOKE_CHECK_TYPE:
                            errorWriter.WriteLine("The set verification method is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_AUTO_IMPORT_CERT_FLAG:
                            errorWriter.WriteLine("The value of auto registration flag of the Certificate is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_SIGN_CONFIG:
                            errorWriter.WriteLine("The setting of the signature is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_IMAGE_FILE:
                            errorWriter.WriteLine("Image file does not exist.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_SIGN_CERT:
                            errorWriter.WriteLine("The selected Certificate can no longer be used.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_SIGN_CERT:
                            errorWriter.WriteLine("The Certificate for signature is not selected.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NOT_USE_PRIVATE_KEY:
                            errorWriter.WriteLine("Access to Private key failed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_INVALID:
                            errorWriter.WriteLine("The Certificate is already revoked.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_SIGN:
                            errorWriter.WriteLine("The signature of the Certificate is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_REVOKE_CHECK_ERROR:
                            errorWriter.WriteLine("Revocation of the Certificate cannot be confirmed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_OUT_OF_VALIDITY:
                            errorWriter.WriteLine("The Certificate is outside the valid date.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_CERT:
                            errorWriter.WriteLine("The Certificate does not exist.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_FAILURE_IMPOPT_CERT:
                            errorWriter.WriteLine("Registration of the Certificate failed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_ROOT_CERT:
                            errorWriter.WriteLine("The Root Certificate does not exist.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_CERT_SIZE:
                            errorWriter.WriteLine("The Certificate size is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_ARG:
                            errorWriter.WriteLine("The argument is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_UNKNOWN:
                            errorWriter.WriteLine("Unknown error.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_OK:
                            errorWriter.WriteLine("The processing succeeded.");
                            break;
                        default:
                            errorWriter.WriteLine("Error.");
                            break;
                    }
                    break;
                default:
                    errorWriter.WriteLine("Unknown signature module.");
                    break;
            }
        }

        // Print usage.
        static void print_usage()
        {
            TextWriter errorWriter = Console.Error;
            errorWriter.WriteLine("Usage : xdwsmpl12 file");
        }
    }
}
