/* xdwapi sample 3

Function :
  Gets the strings of text annotations in the DocuWorks document at the standard output.
How to use:
  xdwsmpl3 document [page]

  document : DocuWorks document (in).
  page : Specified page (in). If omitted, all of pages are covered.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl3
{
    class Program
    {
        static int Main(string[] args)
        {
            int api_result = 0;

            if (args.Length != 2 && args.Length != 1) {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("How to use : xdwsmpl3 document [page]");
                return 0;
            }
            
            string in_path = Path.GetFullPath(args[0]);
            
            int page = 0;
            if (args.Length == 2) {
                page = int.Parse(args[1]);
            }

            // Open the document handle.
            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_READONLY;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Use XDW_GetDocumentInformation to get the number of total pages.
            Xdwapi.XDW_DOCUMENT_INFO info = new Xdwapi.XDW_DOCUMENT_INFO();
            Xdwapi.XDW_GetDocumentInformation(handle, ref info);
            int last_page = info.Pages;
            
            if (args.Length == 2) {
                if (page < 1 || page > last_page) {
                    TextWriter errorWriter = Console.Error;
                    errorWriter.WriteLine("Page not exist.");
                } else {
                    api_result = print_page(handle, page);
                }
            } else if (page == 0) {
                for (int i = 1; i <= last_page; i++) {
                    api_result = print_page(handle, i);
                    if (api_result < 0)
                        break;
                }
            }
            if (api_result < 0)
                print_error(api_result);

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            if (api_result >= 0)
                return 1;

            return 0;
        }

        static int print_page(Xdwapi.XDW_DOCUMENT_HANDLE handle, int page)
        {
            // Use XDW_GetPageInformation to get the number of annotations on the page.
            Xdwapi.XDW_PAGE_INFO_EX pageInfo = new Xdwapi.XDW_PAGE_INFO_EX();
            Xdwapi.XDW_GetPageInformation(handle, page, ref pageInfo);

            int result = 0;
            for (int i = 1; i <= pageInfo.Annotations; i++)
            {
                // Use XDW_GetAnnotationInformation to get information of the annotation.
                Xdwapi.XDW_ANNOTATION_INFO annotationInfo = new Xdwapi.XDW_ANNOTATION_INFO();
                Xdwapi.XDW_GetAnnotationInformation(handle, page, null, i, ref annotationInfo);

                // If annotations are attached on this annotaion,
                if (annotationInfo.ChildAnnotations > 0)
                {
                    for (int j = 1; j <= annotationInfo.ChildAnnotations; j++)
                    {
                        // Use XDW_GetAnnotationInformation to get information of the child annotation.
                        Xdwapi.XDW_ANNOTATION_INFO annotationInfo2 = new Xdwapi.XDW_ANNOTATION_INFO();
                        Xdwapi.XDW_GetAnnotationInformation(handle, page, annotationInfo.Handle, j, ref annotationInfo2);
                        if (annotationInfo2.AnnotationType == Xdwapi.XDW_AID_TEXT)
                        {
                            result = print_text(annotationInfo2.Handle);
                            if (result < 0)
                                break;
                        }
                    }
                }
                else if (annotationInfo.AnnotationType == Xdwapi.XDW_AID_TEXT)
                {
                    result = print_text(annotationInfo.Handle);
                    if (result < 0)
                        break;
                }
            }
            return result;
        }

        static int print_text(Xdwapi.XDW_ANNOTATION_HANDLE annotation)
        {
            // Use XDW_GetAnnotationAttribute to get the attribute value.
            Object text = null;
            int size = Xdwapi.XDW_GetAnnotationAttribute(annotation, Xdwapi.XDW_ATN_Text, ref text);
            Console.WriteLine(text);
            return size;
        }

        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code)
            {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_FILE_EXISTS:
                    errorWriter.WriteLine("File exists.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                default:
                    errorWriter.WriteLine("Error.");
                    break;
            }
        }
    }
}
