/* xdwapi sample 7

Function :
  Creates a new DocuWorks binder and inserts DocuWorks documents into the binder.
How to use :
  xdwsmpl7 binder.xbd [[document.xdw]...]

  binder.xbd : DocuWorks binder (out).
  document.xdw : DocuWorks document (in).

  The color and size of the binder are default values.
  If DocuWorks documents are omitted, creates an empty DocuWorks binder.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl7
{
    class Program
    {
        static int Main(string[] args)
        {
            int api_result = 0;

            if (args.Length < 1) {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("How to use : xdwsmpl7 binder.xbd [[document.xdw]...]");
                return 0;
            }

            string in_path = Path.GetFullPath(args[0]);

            // Create a new binder.
            api_result = Xdwapi.XDW_CreateBinder(in_path, null);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Open the document handle.
            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_UPDATE;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Insert the documents to the binder.
            int doc_num = args.Length - 1;
            for (int i = 1; i <= doc_num; i++) {
                in_path = Path.GetFullPath(args[i]);
                api_result = Xdwapi.XDW_InsertDocumentToBinder(handle, i, in_path);
                if (api_result < 0) {
                    print_error(api_result);
                    break;
                }
            }

            // Reflect the changes to the document.
            if (api_result >= 0) {
                api_result = Xdwapi.XDW_SaveDocument(handle);
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            if (api_result < 0)
                return 0;

            return 1;
        }

        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code)
            {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                default:
                    errorWriter.WriteLine("Error. code: 0x{0}", code.ToString("X"));
                    break;
            }
        }

    }
}
